﻿using Borepin.Base;
using Prism.Commands;
using Prism.Navigation;
using Prism.Services;
using System;
using System.Threading.Tasks;
using System.Windows.Input;
using Xamarin.Forms;
using ZXing;

namespace Borepin.PageModel
{
    class ScanPageModel : PageModelBase
    {
        #region Private Fields
        private object _Instance;
        #endregion

        #region Contructors
        public ScanPageModel(INavigationService navigationService, IPageDialogService pageDialogService) : base(navigationService, pageDialogService)
        {
            AbortCommand = new DelegateCommand(async () => await AbortCommandExecute().ConfigureAwait(true));
            ScannedCommand = new DelegateCommand(ScannedCommandExecute);

            IsVisible = true;
            IsScanning = true;
        }
        #endregion

        #region Data
        public override Task LoadInstance(object instance)
        {
            _Instance = instance;
            return Task.CompletedTask;
        }

        public override Task<object> CreateInstance()
        {
            return Task.FromResult(_Instance);
        }
        #endregion

        #region Fields
        private Result _ScanResult;
        public Result ScanResult
        {
            get => _ScanResult;
            set => SetProperty(ref _ScanResult, value);
        }

        private bool _IsScanning;
        public bool IsScanning
        {
            get => _IsScanning;
            set => SetProperty(ref _IsScanning, value);
        }

        private bool _IsVisible;
        public bool IsVisible
        {
            get => _IsVisible;
            set => SetProperty(ref _IsVisible, value);
        }
        #endregion

        #region Commands
        private ICommand _ScannedCommand;

        public ICommand ScannedCommand
        {
            get => _ScannedCommand;
            set => SetProperty(ref _ScannedCommand, value);
        }
        public void ScannedCommandExecute()
        {
            IsScanning = false;
            IsVisible = false;

            INavigationParameters parameters = new NavigationParameters()
            {
                { "result", "scanned" },
                { "value", ScanResult.Text },
            };

            Device.BeginInvokeOnMainThread(async () =>
            {
                INavigationResult result = await _NavigationService.GoBackAsync(parameters).ConfigureAwait(false);
            });
        }

        private ICommand _AbortCommand;
        public ICommand AbortCommand
        {
            get => _AbortCommand;
            set => SetProperty(ref _AbortCommand, value);
        }
        public async Task AbortCommandExecute()
        {
            IsScanning = false;
            IsVisible = false;

            INavigationParameters parameters = new NavigationParameters()
            {
                { "result", "abort" },
            };
            await _NavigationService.GoBackAsync(parameters).ConfigureAwait(false);
        }
        #endregion
    }
}
